IMPLEMENTATION MODULE ConsoleDevice;

(* Die Anleitungen und Erluterungen befinden sich im Definitionsfile  *)
(* Compiler M2Amiga V4.097d                   1991 by Andre Wiethoff  *)


(*$ StackChk:=FALSE *)
(*$ RangeChk:=FALSE *)
(*$ OverflowChk:=FALSE *)
(*$ NilChk:=FALSE *)
(*$ CaseChk:=FALSE *)
(*$ ReturnChk:=FALSE *)
(*$ Volatile:=FALSE *)


FROM SYSTEM       IMPORT ADR,LONGSET,ADDRESS;
FROM ExecD        IMPORT MsgPortPtr,TaskPtr,MsgPort,IOStdReq,read,write;
FROM ExecL        IMPORT OpenDevice,AddPort,FindTask,RemPort,CloseDevice,DoIO,
                         GetMsg,SendIO;
FROM IntuitionD   IMPORT WindowPtr,IntuiMessagePtr;
FROM RememberHeap IMPORT NewRememberPtr,NewAllocRemember,NewFreeRemember,
                         CutRememberStructure,GetAddress;
FROM String       IMPORT Length;

VAR
  task            : TaskPtr;
  command         : ARRAY[0..10] OF CHAR;
  rememberConsole : NewRememberPtr;
  message         : IntuiMessagePtr;


PROCEDURE OpenConsole(window : WindowPtr) : ConsoleHandlePtr;
VAR ch : ConsoleHandlePtr;
BEGIN
  task:=FindTask(0);
  ch:=NewAllocRemember(rememberConsole,SIZE(ConsoleHandle),FALSE);
  IF ch#NIL THEN
    WITH ch^ DO
      msgPortRead.sigTask:=task;
      msgPortWrite.sigTask:=task;
      AddPort(ADR(msgPortRead));
      AddPort(ADR(msgPortWrite));
      ioReqRead.data:=ADDRESS(window);
      ioReqRead.length:=SIZE(window^);
      OpenDevice(ADR("console.device"),0,ADR(ioReqRead),LONGSET{});
      IF (ioReqRead.device#NIL) AND (ioReqRead.error=0) THEN
        ioReqRead.message.replyPort:=ADR(msgPortRead);
        ioReqWrite.message.replyPort:=ADR(msgPortWrite);
        ioReqWrite.device:=ioReqRead.device;
        ioReqWrite.unit:=ioReqRead.unit;
        conUnit:=ADDRESS(ioReqRead.unit);
        charIn:=CHR(0);
        ioReqRead.command:=read;
        ioReqRead.data:=ADR(charIn);
        ioReqRead.length:=1;
        SendIO(ADR(ioReqRead));
      ELSE
        RemPort(ADR(msgPortWrite));
        RemPort(ADR(msgPortRead));
        CutRememberStructure(rememberConsole,ch,TRUE);
        ch:=NIL;
      END;
    END;
  END;
  RETURN ch;
END OpenConsole;



PROCEDURE ResetConsole(console : ConsoleHandlePtr);
BEGIN
  IF console#NIL THEN
    command[0]:=csi; command[1]:='c'; command[2]:=CHR(0);
    Write(console,command);
  END;
END ResetConsole;



PROCEDURE Write(console : ConsoleHandlePtr;
                string  : ARRAY OF CHAR);
BEGIN
  IF console#NIL THEN
    WITH console^ DO
      ioReqWrite.command:=write;
      ioReqWrite.data:=ADR(string);
      ioReqWrite.length:=Length(string);
      DoIO(ADR(ioReqWrite));
    END;
  END;
END Write;



PROCEDURE WriteChar(console : ConsoleHandlePtr;
                    char    : CHAR);
BEGIN
  IF console#NIL THEN
    WITH console^ DO
      ioReqWrite.command:=write;
      ioReqWrite.data:=ADR(char);
      ioReqWrite.length:=1;
      DoIO(ADR(ioReqWrite));
    END;
  END;
END WriteChar;



PROCEDURE MoveCursor(console          : ConsoleHandlePtr;
                     number,direction : INTEGER);
BEGIN
  IF (direction<4) AND (number<256) AND (direction>=0) AND (number>=0) THEN
    command[0]:=csi; command[1]:=CHR((number DIV 100)+48);
    command[2]:=CHR(((number DIV 10) MOD 10)+48);
    command[3]:=CHR((number MOD 10)+48);
    command[4]:=CHR(65+direction); command[5]:=CHR(0); Write(console,command);
  END;
END MoveCursor;



PROCEDURE Cursor(console : ConsoleHandlePtr; flag : BOOLEAN);
BEGIN
  command[0]:=csi;
  IF flag THEN
    command[1]:=' '; command[2]:='p'; command[3]:=CHR(0);
  ELSE
    command[1]:='0'; command[2]:=' '; command[3]:='p'; command[4]:=CHR(0);
  END;
  Write(console,command);
END Cursor;



PROCEDURE SetCursor(console : ConsoleHandlePtr;
                    x,y     : INTEGER);
BEGIN
  IF (x<100) AND (y<100) AND (x>=0) AND (y>=0) THEN
    command[0]:=csi; command[1]:=CHR((y DIV 10)+48);
    command[2]:=CHR((y MOD 10)+48); command[3]:=';';
    command[4]:=CHR(48+(x DIV 10)); command[5]:=CHR(48+(x MOD 10));
    command[6]:='H'; command[7]:=CHR(0);
    Write(console,command);
  END;
END SetCursor;



PROCEDURE GetCursorPos(console : ConsoleHandlePtr;
                       VAR x,y : INTEGER);
BEGIN
  IF console#NIL THEN
    x:=console^.conUnit^.xcCP; y:=console^.conUnit^.ycCP;
  END;
END GetCursorPos;



PROCEDURE InsertLine(console : ConsoleHandlePtr);
BEGIN
  command[0]:=csi; command[1]:='L'; command[2]:=CHR(0);
  Write(console,command);
END InsertLine;



PROCEDURE InsertSpaces(console : ConsoleHandlePtr; number : INTEGER);
BEGIN
  IF (number<100) AND (number>=0) THEN
    command[0]:=csi; command[1]:=CHR((number DIV 10)+48);
    command[2]:=CHR((number MOD 10)+48); command[3]:=CHR(64);
    command[4]:=CHR(0);
    Write(console,command);
  END;
END InsertSpaces;



PROCEDURE DeleteLine(console : ConsoleHandlePtr);
BEGIN
  command[0]:=csi; command[1]:='M'; command[2]:=CHR(0);
  Write(console,command);
END DeleteLine;



PROCEDURE DeleteChars(console : ConsoleHandlePtr;
                      number  : INTEGER);
BEGIN
  IF (number<100) AND (number>=0) THEN
    command[0]:=csi; command[1]:=CHR(48+(number DIV 10));
    command[2]:=CHR(48+(number MOD 10)); command[3]:='P'; command[4]:=CHR(0);
    Write(console,command);
  END;
END DeleteChars;



PROCEDURE Erase(console : ConsoleHandlePtr;
                eo      : INTEGER);
BEGIN
  IF (eo>=EOD) AND (eo<=EOL) THEN
    command[0]:=csi; command[1]:=CHR(75+eo); command[2]:=CHR(0);
    Write(console,command);
  END;
END Erase;



PROCEDURE Scroll(console : ConsoleHandlePtr;
                 number  : INTEGER);
BEGIN
  IF (number#0) AND (number<50) AND (number>-50) THEN
    command[0]:=csi; command[1]:=CHR(48+(ABS(number) DIV 10));
    command[2]:=CHR(48+(ABS(number) MOD 10)); command[4]:=CHR(0);
    IF number<0 THEN
      command[3]:='T';
    ELSE
      command[3]:='S';
    END;
  END;
  Write(console,command);
END Scroll;



PROCEDURE SetStyle(console             : ConsoleHandlePtr;
                   style,foregr,backgr : INTEGER);
BEGIN
  IF (backgr<8) AND (foregr<8) AND (backgr>=0) AND (foregr>=0) THEN
    IF (style=0) OR (style=1) OR (style=3) OR (style=4) OR (style=7) THEN
      command[0]:=csi; command[1]:=CHR(style+48); command[2]:=';';
      command[3]:='3'; command[4]:=CHR(foregr+48); command[5]:=';';
      command[6]:='4'; command[7]:=CHR(backgr+48); command[8]:='m';
      command[9]:=CHR(0);
      Write(console,command);
    END;
  END;
END SetStyle;



PROCEDURE Set(console          : ConsoleHandlePtr;
              qualifier,number : INTEGER);
VAR t : ARRAY[0..3] OF CHAR;
BEGIN
  t[0]:='t'; t[1]:='u'; t[2]:='x'; t[3]:='y';
  IF (qualifier<4) AND (qualifier>=0) AND (number<1000) AND (number>=0) THEN
    command[0]:=csi;
    IF number#0 THEN
      command[1]:=CHR(48+(number DIV 100));
      command[2]:=CHR(48+((number DIV 10) MOD 10));
      command[3]:=CHR(48+(number MOD 10)); command[4]:=t[qualifier]; command[5]:=CHR(0);
    ELSE
      command[1]:=t[qualifier]; command[2]:=CHR(0);
    END;
    Write(console,command);
  END;
END Set;



PROCEDURE GetSize(console : ConsoleHandlePtr;
                  VAR x,y : INTEGER);
BEGIN
  IF console#NIL THEN
    x:=console^.conUnit^.xMax+1; y:=console^.conUnit^.yMax+1;
  END;
END GetSize;



PROCEDURE ReadForm(console   : ConsoleHandlePtr;
                   VAR input : Form);
VAR z : INTEGER;
BEGIN
  IF console#NIL THEN
    WITH console^ DO
      FOR z:=0 TO 49 DO
        input[z]:=CHR(0);
      END;
      z:=0;
      message:=GetMsg(ADR(msgPortRead));
      IF message#NIL THEN
        input[0]:=charIn;
        IF input[0]=csi THEN
          REPEAT
            INC(z);
            ioReqRead.command:=read;
            ioReqRead.length:=1;
            ioReqRead.data:=ADR(input[z]);
            DoIO(ADR(ioReqRead));
          UNTIL (((input[z]<'0') OR (input[z]>'9')) AND NOT ((input[z]='?') AND
	        (z=1)) AND (input[z]#';')) OR (z>48);
        END;
        charIn:=CHR(0);
        ioReqRead.command:=read;
        ioReqRead.data:=ADR(charIn);
        ioReqRead.length:=1;
        ioReqRead.message.replyPort:=ADR(msgPortRead);
        SendIO(ADR(ioReqRead));
      END;
    END;
  END;
END ReadForm;



PROCEDURE Inkey(console : ConsoleHandlePtr) : CHAR;
VAR gf : Form;
BEGIN
  IF console#NIL THEN
    ReadForm(console,gf);
  END;
  RETURN gf[0];
END Inkey;



PROCEDURE Read(console    : ConsoleHandlePtr;
	       maxC       : INTEGER;
	       VAR string : ARRAY OF CHAR);
VAR z : INTEGER;
    f : Form;
BEGIN
  IF console#NIL THEN  (* Sicherheitsabfrage ob geffnet *)
    z:=0;              (* Zeichennummer = 0		 *)
    REPEAT
      ReadForm(console,f); (* Mgliches ersten Zeichen lesen *)

      IF (ORD(f[0])=8) AND (z>0) THEN
         (* Backspacetaste gedrckt! *)
         WriteChar(console,f[0]); DeleteChars(console,1);
         DEC(z); (* Zhler erniedrigen *)
      END;
      IF (z<HIGH(string)) AND ((z<maxC) OR (maxC=0)) AND
      (* Zhler noch nicht zu gro UND ... *)
      (((f[0]>=' ') AND (ORD(f[0])<128)) OR (ORD(f[0])>=160)) THEN
        (* Zeichen liegt zwischen ASCII 32 und 128 ODER ber 160 *)
        (* dann soll das Zeichen ausgegeben werden. *)
        WriteChar(console,f[0]); string[z]:=f[0]; INC(z);
      END;
    UNTIL (ORD(f[0])=13); (* Solange wiederholen, bis RETURN gedrckt wurde *)
    Write(console,ln);    (* Zeilenvorschub ausgeben (kein CR!!)	    *)
    string[z]:=CHR(0);    (* String mit NULL-Kennung abschlieen.	    *)
  END;
END Read;



PROCEDURE CloseConsole(VAR console : ConsoleHandlePtr);
BEGIN
  IF console#NIL THEN
    WITH console^ DO
      RemPort(ADR(msgPortRead));
      RemPort(ADR(msgPortWrite));
      CloseDevice(ADR(ioReqRead));
    END;
    CutRememberStructure(rememberConsole,console,TRUE);
    console:=NIL;
  END;
END CloseConsole;



VAR rem : NewRememberPtr;
    cc  : ConsoleHandlePtr;

BEGIN

  rememberConsole:=NIL;

CLOSE

  rem:=rememberConsole;
  WHILE rem#NIL DO
    cc:=GetAddress(rem);
    CloseConsole(cc);
    rem:=rem^.next;
  END;
  NewFreeRemember(rememberConsole,TRUE);

END ConsoleDevice.
